@@ -26,6 +26,7 @@ extern "C" {
 #include <cstdint>
 #include <utility>
 #include <vector>
+#include <unordered_set>
 
 #include <Eigen/Core>
 #include <Eigen/LU>
@@ -43,8 +44,6 @@ extern "C" {
 #include <srt/srt.h>
 #endif
 
-#define FRAME_SIZE (8 << 20)  // 8 MB.
-
 using namespace std;
 using namespace std::chrono;
 using namespace bmusb;
@@ -454,24 +453,63 @@ void FFmpegCapture::send_disconnected_frame()
 	}
 }
 
-AVPixelFormat get_vaapi_hw_format(AVCodecContext *ctx, const AVPixelFormat *fmt)
+template<AVHWDeviceType type>
+AVPixelFormat get_hw_format(AVCodecContext *ctx, const AVPixelFormat *fmt)
 {
-	for (const AVPixelFormat *fmt_ptr = fmt; *fmt_ptr != -1; ++fmt_ptr) {
-		for (int i = 0;; ++i) {  // Termination condition inside loop.
-			const AVCodecHWConfig *config = avcodec_get_hw_config(ctx->codec, i);
-			if (config == nullptr) {  // End of list.
-				fprintf(stderr, "Decoder %s does not support device.\n", ctx->codec->name);
-				break;
-			}
-			if (config->methods & AV_CODEC_HW_CONFIG_METHOD_HW_DEVICE_CTX &&
-			    config->device_type == AV_HWDEVICE_TYPE_VAAPI &&
-			    config->pix_fmt == *fmt_ptr) {
+	bool found_config_of_right_type = false;
+	for (int i = 0;; ++i) {  // Termination condition inside loop.
+		const AVCodecHWConfig *config = avcodec_get_hw_config(ctx->codec, i);
+		if (config == nullptr) {  // End of list.
+			break;
+		}
+		if (!(config->methods & AV_CODEC_HW_CONFIG_METHOD_HW_DEVICE_CTX) ||
+		    config->device_type != type) {
+			// Not interesting for us.
+			continue;
+		}
+
+		// We have a config of the right type, but does it actually support
+		// the pixel format we want? (Seemingly, FFmpeg's way of signaling errors
+		// is to just replace the pixel format with a software-decoded one,
+		// such as yuv420p.)
+		found_config_of_right_type = true;
+		for (const AVPixelFormat *fmt_ptr = fmt; *fmt_ptr != -1; ++fmt_ptr) {
+			if (config->pix_fmt == *fmt_ptr) {
+				fprintf(stderr, "Initialized '%s' hardware decoding for codec '%s'.\n",
+					av_hwdevice_get_type_name(type), ctx->codec->name);
+				if (ctx->profile == FF_PROFILE_H264_BASELINE) {
+					fprintf(stderr, "WARNING: Stream claims to be H.264 Baseline, which is generally poorly supported in hardware decoders.\n");
+					fprintf(stderr, "         Consider encoding it as Constrained Baseline, Main or High instead.\n");
+					fprintf(stderr, "         Decoding might fail and fall back to software.\n");
+				}
 				return config->pix_fmt;
 			}
 		}
+		fprintf(stderr, "Decoder '%s' supports only these pixel formats:", ctx->codec->name);
+		unordered_set<AVPixelFormat> seen;
+		for (const AVPixelFormat *fmt_ptr = fmt; *fmt_ptr != -1; ++fmt_ptr) {
+			if (!seen.count(*fmt_ptr)) {
+				fprintf(stderr, " %s", av_get_pix_fmt_name(*fmt_ptr));
+				seen.insert(*fmt_ptr);
+			}
+		}
+		fprintf(stderr, " (wanted %s for hardware acceleration)\n", av_get_pix_fmt_name(config->pix_fmt));
+
 	}
 
-	// We found no VA-API formats, so take the best software format.
+	if (!found_config_of_right_type) {
+		fprintf(stderr, "Decoder '%s' does not support device type '%s'.\n", ctx->codec->name, av_hwdevice_get_type_name(type));
+	}
+
+	// We found no VA-API formats, so take the first software format.
+	for (const AVPixelFormat *fmt_ptr = fmt; *fmt_ptr != -1; ++fmt_ptr) {
+		if ((av_pix_fmt_desc_get(*fmt_ptr)->flags & AV_PIX_FMT_FLAG_HWACCEL) == 0) {
+			fprintf(stderr, "Falling back to software format %s.\n", av_get_pix_fmt_name(*fmt_ptr));
+			return *fmt_ptr;
+		}
+	}
+
+	// Fallback: Just return anything. (Should never really happen.)
 	return fmt[0];
 }
 
@@ -544,14 +582,24 @@ bool FFmpegCapture::play_video(const string &pathname)
 	}
 
 	// Seemingly, it's not too easy to make something that just initializes
-	// “whatever goes”, so we don't get VDPAU or CUDA here without enumerating
-	// through several different types. VA-API will do for now.
+	// “whatever goes”, so we don't get CUDA or VULKAN or whatever here
+	// without enumerating through several different types.
+	// VA-API and VDPAU will do for now. We prioritize VDPAU for the
+	// simple reason that there's a VA-API-via-VDPAU emulation for NVidia
+	// cards that seems to work, but just hangs when trying to transfer the frame.
+	//
+	// Note that we don't actually check codec support beforehand,
+	// so if you have a low-end VDPAU device but a high-end VA-API device,
+	// you lose out on the extra codec support from the latter.
 	AVBufferRef *hw_device_ctx = nullptr;
-	if (av_hwdevice_ctx_create(&hw_device_ctx, AV_HWDEVICE_TYPE_VAAPI, nullptr, nullptr, 0) < 0) {
-		fprintf(stderr, "Failed to initialize VA-API for FFmpeg acceleration. Decoding video in software.\n");
-	} else {
+	if (av_hwdevice_ctx_create(&hw_device_ctx, AV_HWDEVICE_TYPE_VDPAU, nullptr, nullptr, 0) >= 0) {
 		video_codec_ctx->hw_device_ctx = av_buffer_ref(hw_device_ctx);
-		video_codec_ctx->get_format = get_vaapi_hw_format;
+		video_codec_ctx->get_format = get_hw_format<AV_HWDEVICE_TYPE_VDPAU>;
+	} else if (av_hwdevice_ctx_create(&hw_device_ctx, AV_HWDEVICE_TYPE_VAAPI, nullptr, nullptr, 0) >= 0) {
+		video_codec_ctx->hw_device_ctx = av_buffer_ref(hw_device_ctx);
+		video_codec_ctx->get_format = get_hw_format<AV_HWDEVICE_TYPE_VAAPI>;
+	} else {
+		fprintf(stderr, "Failed to initialize VA-API or VDPAU for FFmpeg acceleration. Decoding video in software.\n");
 	}
 
 	if (avcodec_open2(video_codec_ctx.get(), video_codec, nullptr) < 0) {
@@ -591,6 +639,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 
 	// Main loop.
 	bool first_frame = true;
+	int consecutive_errors = 0;
 	while (!producer_thread_should_quit.should_quit()) {
 		if (process_queued_commands(format_ctx.get(), pathname, last_modified, /*rewound=*/nullptr)) {
 			return true;
@@ -607,7 +656,14 @@ bool FFmpegCapture::play_video(const string &pathname)
 		AVFrameWithDeleter frame = decode_frame(format_ctx.get(), video_codec_ctx.get(), audio_codec_ctx.get(),
 			pathname, video_stream_index, audio_stream_index, subtitle_stream_index, audio_frame.get(), &audio_format, &audio_pts, &error);
 		if (error) {
-			return false;
+			if (++consecutive_errors >= 100) {
+				fprintf(stderr, "More than 100 consecutive video frames, aborting playback.\n");
+				return false;
+			} else {
+				continue;
+			}
+		} else {
+			consecutive_errors = 0;
 		}
 		if (frame == nullptr) {
 			// EOF. Loop back to the start if we can.
@@ -715,7 +771,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 						1e3 * duration<double>(now - next_frame_start).count());
 					pts_origin = frame->pts;
 					start = next_frame_start = now;
-					timecode += MAX_FPS * 2 + 1;
+					timecode += TYPICAL_FPS * 2 + 1;
 				}
 			}
 			bool finished_wakeup;
@@ -733,7 +789,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 					// Make sure to get the audio resampler reset. (This is a hack;
 					// ideally, the frame callback should just accept a way to signal
 					// audio discontinuity.)
-					timecode += MAX_FPS * 2 + 1;
+					timecode += TYPICAL_FPS * 2 + 1;
 				}
 				last_neutral_color = get_neutral_color(frame->metadata);
 				if (frame_callback != nullptr) {
@@ -891,7 +947,8 @@ AVFrameWithDeleter FFmpegCapture::decode_frame(AVFormatContext *format_ctx, AVCo
 		// Decode video, if we have a frame.
 		int err = avcodec_receive_frame(video_codec_ctx, video_avframe.get());
 		if (err == 0) {
-			if (video_avframe->format == AV_PIX_FMT_VAAPI) {
+			if (video_avframe->format == AV_PIX_FMT_VAAPI ||
+			    video_avframe->format == AV_PIX_FMT_VDPAU) {
 				// Get the frame down to the CPU. (TODO: See if we can keep it
 				// on the GPU all the way, since it will be going up again later.
 				// However, this only works if the OpenGL GPU is the same one.)
@@ -1087,6 +1144,16 @@ UniqueFrame FFmpegCapture::make_video_frame(const AVFrame *frame, const string &
 
 		current_frame_ycbcr_format = decode_ycbcr_format(desc, frame, is_mjpeg, &last_colorspace, &last_chroma_location);
 	}
+
+	// FIXME: Currently, if the video is too high-res for one of the allocated
+	// frames, we simply refuse to scale it here to avoid crashes. It would be better
+	// if we could somehow signal getting larger frames, especially as 4K is a thing now.
+	if (video_frame->len > FRAME_SIZE) {
+		fprintf(stderr, "%s: Decoded frame would be larger than supported FRAME_SIZE (%zu > %u), not decoding.\n", pathname.c_str(), video_frame->len, FRAME_SIZE);
+		*error = true;
+		return video_frame;
+	}
+
 	sws_scale(sws_ctx.get(), frame->data, frame->linesize, 0, frame->height, pic_data, linesizes);
 
 	return video_frame;
