@@ -159,9 +159,13 @@ public:
 			std::placeholders::_11);
 	}
 
-	// FFmpegCapture-specific callback that gives the raw audio.
-	typedef std::function<void(const AVPacket *pkt, const AVRational timebase)> audio_callback_t;
-	void set_audio_callback(audio_callback_t callback)
+	// FFmpegCapture-specific callback that gives the raw audio/video.
+	typedef std::function<void(const AVPacket *pkt, const AVRational timebase)> packet_callback_t;
+	void set_video_callback(packet_callback_t callback)
+	{
+		video_callback = callback;
+	}
+	void set_audio_callback(packet_callback_t callback)
 	{
 		audio_callback = callback;
 	}
@@ -297,7 +301,8 @@ private:
 	std::unique_ptr<bmusb::FrameAllocator> owned_video_frame_allocator;
 	std::unique_ptr<bmusb::FrameAllocator> owned_audio_frame_allocator;
 	frame_callback_t frame_callback = nullptr;
-	audio_callback_t audio_callback = nullptr;
+	packet_callback_t video_callback = nullptr;
+	packet_callback_t audio_callback = nullptr;
 
 	SwsContextWithDeleter sws_ctx;
 	int sws_last_width = -1, sws_last_height = -1, sws_last_src_format = -1;
