@@ -23,6 +23,7 @@ enum LongOption {
 	OPTION_V4L_OUTPUT,
 	OPTION_HTTP_UNCOMPRESSED_VIDEO,
 	OPTION_HTTP_X264_VIDEO,
+	OPTION_HTTP_AV1_VIDEO,
 	OPTION_RECORD_X264_VIDEO,
 	OPTION_SEPARATE_X264_DISK_ENCODE,
 	OPTION_X264_PRESET,
@@ -39,6 +40,10 @@ enum LongOption {
 	OPTION_X264_SEPARATE_DISK_BITRATE,
 	OPTION_X264_SEPARATE_DISK_CRF,
 	OPTION_X264_SEPARATE_DISK_PARAM,
+	OPTION_AV1_PRESET,
+	OPTION_AV1_BITRATE,
+	OPTION_AV1_FPS,
+	OPTION_AV1_PARAM,
 	OPTION_HTTP_MUX,
 	OPTION_HTTP_COARSE_TIMEBASE,
 	OPTION_HTTP_AUDIO_CODEC,
@@ -73,6 +78,7 @@ enum LongOption {
 	OPTION_TIMECODE_STREAM,
 	OPTION_TIMECODE_STDOUT,
 	OPTION_QUICK_CUT_KEYS,
+	OPTION_10_BIT,
 	OPTION_10_BIT_INPUT,
 	OPTION_10_BIT_OUTPUT,
 	OPTION_INPUT_YCBCR_INTERPRETATION,
@@ -156,8 +162,7 @@ void usage(Program program)
 		fprintf(stderr, "      --midi-mapping=FILE         start with the given MIDI controller mapping (implies --multichannel)\n");
 		fprintf(stderr, "      --default-hdmi-input        default to HDMI over SDI inputs for cards that have both\n");
 		fprintf(stderr, "      --fake-cards-audio          make fake (disconnected) cards output a simple tone\n");
-		fprintf(stderr, "      --v4l-output DEVICE         send video (no audio) to V4L2 output/loopback\n");
-		fprintf(stderr, "      --http-uncompressed-video   send uncompressed NV12 video to HTTP clients\n");
+		fprintf(stderr, "      --v4l-output=DEVICE         send video (no audio) to V4L2 output/loopback\n");
 		fprintf(stderr, "      --http-x264-video           send x264-compressed video to HTTP clients\n");
 		fprintf(stderr, "      --record-x264-video         store x264-compressed video to disk (implies --http-x264-video,\n");
 		fprintf(stderr, "                                    removes the need for working VA-API encoding)\n");
@@ -167,14 +172,9 @@ void usage(Program program)
 	fprintf(stderr, "      --x264-preset               x264 quality preset (default " X264_DEFAULT_PRESET ")\n");
 	fprintf(stderr, "      --x264-tune                 x264 tuning (default " X264_DEFAULT_TUNE ", can be blank)\n");
 	fprintf(stderr, "      --x264-speedcontrol         try to match x264 preset to available CPU speed\n");
-	fprintf(stderr, "      --x264-speedcontrol-verbose  output speedcontrol debugging statistics\n");
 	fprintf(stderr, "      --x264-bitrate              x264 bitrate (in kilobit/sec, default %d)\n",
 		DEFAULT_X264_OUTPUT_BIT_RATE);
-	fprintf(stderr, "      --x264-crf=VALUE            quality-based VBR (-12 to 51), incompatible with --x264-bitrate and VBV\n");
-	fprintf(stderr, "      --x264-vbv-bufsize          x264 VBV size (in kilobits, 0 = one-frame VBV,\n");
-	fprintf(stderr, "                                  default: same as --x264-bitrate, that is, one-second VBV)\n");
-	fprintf(stderr, "      --x264-vbv-max-bitrate      x264 local max bitrate (in kilobit/sec per --vbv-bufsize,\n");
-	fprintf(stderr, "                                  0 = no limit, default: same as --x264-bitrate, i.e., CBR)\n");
+	fprintf(stderr, "      --x264-crf=VALUE            quality-based VBR (-12 to 51), incompatible with --x264-bitrate\n");
 	fprintf(stderr, "      --x264-param=NAME[,VALUE]   set any x264 parameter, for fine tuning\n");
 	if (program == PROGRAM_NAGERU) {
 		fprintf(stderr, "      --x264-separate-disk-preset x264 quality preset (default " X264_DEFAULT_PRESET ")\n");
@@ -185,6 +185,16 @@ void usage(Program program)
 		fprintf(stderr, "                                  incompatible with --x264-separate-disk-bitrate\n");
 		fprintf(stderr, "      --x264-separate-disk-param=NAME[,VALUE] set any x264 parameter, for fine tuning\n");
 	}
+#ifdef HAVE_AV1
+	fprintf(stderr, "      --http-av1-video            send AV1-compressed video to HTTP clients\n");
+	fprintf(stderr, "      --av1-preset                SVT-AV1 quality preset (default %d, from -2 to 13)\n",
+		DEFAULT_AV1_PRESET);
+	fprintf(stderr, "      --av1-bitrate               AV1 bitrate (in kilobit/sec, default %d)\n",
+		DEFAULT_AV1_OUTPUT_BIT_RATE);
+	fprintf(stderr, "      --av1-fps=NUM[/DEN]         AV1 encoded frame rate (default %d/%d)\n",
+		DEFAULT_AV1_FPS_NUM, DEFAULT_AV1_FPS_DEN);
+	fprintf(stderr, "      --av1-param=NAME[,VALUE]    set any SVT-AV1 parameter, for fine tuning\n");
+#endif
 	fprintf(stderr, "      --http-mux=NAME             mux to use for HTTP streams (default " DEFAULT_STREAM_MUX_NAME ")\n");
 	fprintf(stderr, "      --http-audio-codec=NAME     audio codec to use for HTTP streams\n");
 	fprintf(stderr, "                                  (default is to use the same as for the recording)\n");
@@ -214,9 +224,6 @@ void usage(Program program)
 		fprintf(stderr, "      --disable-limiter           turn off limiter (also --enable)\n");
 		fprintf(stderr, "      --disable-makeup-gain-auto  turn off auto-adjustment of final makeup gain (also --enable)\n");
 		fprintf(stderr, "      --disable-alsa-output       disable audio monitoring via ALSA\n");
-		fprintf(stderr, "      --no-flush-pbos             do not explicitly signal texture data uploads\n");
-		fprintf(stderr, "                                    (will give display corruption, but makes it\n");
-		fprintf(stderr, "                                    possible to run with apitrace in real time)\n");
 		fprintf(stderr, "      --print-video-latency       print out measurements of video latency on stdout\n");
 		fprintf(stderr, "      --max-input-queue-frames=FRAMES  never keep more than FRAMES frames for each card\n");
 		fprintf(stderr, "                                    (default 6, minimum 1)\n");
@@ -236,9 +243,7 @@ void usage(Program program)
 		fprintf(stderr, "      --timecode-stream           show timestamp and timecode in stream\n");
 		fprintf(stderr, "      --timecode-stdout           show timestamp and timecode on standard output\n");
 		fprintf(stderr, "      --quick-cut-keys            enable direct cutting by Q, W, E, ... keys\n");
-		fprintf(stderr, "      --10-bit-input              use 10-bit video input (requires compute shaders)\n");
-		fprintf(stderr, "      --10-bit-output             use 10-bit video output (requires compute shaders,\n");
-		fprintf(stderr, "                                    implies --record-x264-video)\n");
+		fprintf(stderr, "      --10-bit                    use 10-bit color depth\n");
 		fprintf(stderr, "      --input-ycbcr-interpretation=CARD,{rec601,rec709,auto}[,{limited,full}]\n");
 		fprintf(stderr, "                                  Y'CbCr coefficient standard of card CARD (default auto)\n");
 		fprintf(stderr, "                                    auto is rec601 for SD, rec709 for HD, always limited\n");
@@ -271,24 +276,26 @@ void parse_flags(Program program, int argc, char * const argv[])
 		{ "default-hdmi-input", no_argument, 0, OPTION_DEFAULT_HDMI_INPUT },
 		{ "fake-cards-audio", no_argument, 0, OPTION_FAKE_CARDS_AUDIO },
 		{ "v4l-output", required_argument, 0, OPTION_V4L_OUTPUT },
-		{ "http-uncompressed-video", no_argument, 0, OPTION_HTTP_UNCOMPRESSED_VIDEO },
 		{ "http-x264-video", no_argument, 0, OPTION_HTTP_X264_VIDEO },
 		{ "record-x264-video", no_argument, 0, OPTION_RECORD_X264_VIDEO },
 		{ "separate-x264-disk-encode", no_argument, 0, OPTION_SEPARATE_X264_DISK_ENCODE },
 		{ "x264-preset", required_argument, 0, OPTION_X264_PRESET },
 		{ "x264-tune", required_argument, 0, OPTION_X264_TUNE },
 		{ "x264-speedcontrol", no_argument, 0, OPTION_X264_SPEEDCONTROL },
-		{ "x264-speedcontrol-verbose", no_argument, 0, OPTION_X264_SPEEDCONTROL_VERBOSE },
-		{ "x264-bitrate", required_argument, 0, OPTION_X264_BITRATE },
 		{ "x264-crf", required_argument, 0, OPTION_X264_CRF },
-		{ "x264-vbv-bufsize", required_argument, 0, OPTION_X264_VBV_BUFSIZE },
-		{ "x264-vbv-max-bitrate", required_argument, 0, OPTION_X264_VBV_MAX_BITRATE },
 		{ "x264-param", required_argument, 0, OPTION_X264_PARAM },
 		{ "x264-separate-disk-preset", required_argument, 0, OPTION_X264_SEPARATE_DISK_PRESET },
 		{ "x264-separate-disk-tune", required_argument, 0, OPTION_X264_SEPARATE_DISK_TUNE },
 		{ "x264-separate-disk-bitrate", required_argument, 0, OPTION_X264_SEPARATE_DISK_BITRATE },
 		{ "x264-separate-disk-crf", required_argument, 0, OPTION_X264_SEPARATE_DISK_CRF },
 		{ "x264-separate-disk-param", required_argument, 0, OPTION_X264_SEPARATE_DISK_PARAM },
+#ifdef HAVE_AV1
+		{ "http-av1-video", no_argument, 0, OPTION_HTTP_AV1_VIDEO },
+		{ "av1-preset", required_argument, 0, OPTION_AV1_PRESET },
+		{ "av1-bitrate", required_argument, 0, OPTION_AV1_BITRATE },
+		{ "av1-fps", required_argument, 0, OPTION_AV1_FPS },
+		{ "av1-param", required_argument, 0, OPTION_AV1_PARAM },
+#endif
 		{ "http-mux", required_argument, 0, OPTION_HTTP_MUX },
 		{ "http-audio-codec", required_argument, 0, OPTION_HTTP_AUDIO_CODEC },
 		{ "http-audio-bitrate", required_argument, 0, OPTION_HTTP_AUDIO_BITRATE },
@@ -311,7 +318,6 @@ void parse_flags(Program program, int argc, char * const argv[])
 		{ "disable-makeup-gain-auto", no_argument, 0, OPTION_DISABLE_MAKEUP_GAIN_AUTO },
 		{ "enable-makeup-gain-auto", no_argument, 0, OPTION_ENABLE_MAKEUP_GAIN_AUTO },
 		{ "disable-alsa-output", no_argument, 0, OPTION_DISABLE_ALSA_OUTPUT },
-		{ "no-flush-pbos", no_argument, 0, OPTION_NO_FLUSH_PBOS },
 		{ "print-video-latency", no_argument, 0, OPTION_PRINT_VIDEO_LATENCY },
 		{ "max-input-queue-frames", required_argument, 0, OPTION_MAX_INPUT_QUEUE_FRAMES },
 		{ "audio-queue-length-ms", required_argument, 0, OPTION_AUDIO_QUEUE_LENGTH_MS },
@@ -322,10 +328,24 @@ void parse_flags(Program program, int argc, char * const argv[])
 		{ "timecode-stream", no_argument, 0, OPTION_TIMECODE_STREAM },
 		{ "timecode-stdout", no_argument, 0, OPTION_TIMECODE_STDOUT },
 		{ "quick-cut-keys", no_argument, 0, OPTION_QUICK_CUT_KEYS },
-		{ "10-bit-input", no_argument, 0, OPTION_10_BIT_INPUT },
-		{ "10-bit-output", no_argument, 0, OPTION_10_BIT_OUTPUT },
+		{ "10-bit", no_argument, 0, OPTION_10_BIT },
 		{ "input-ycbcr-interpretation", required_argument, 0, OPTION_INPUT_YCBCR_INTERPRETATION },
 		{ "mjpeg-export-cards", required_argument, 0, OPTION_MJPEG_EXPORT_CARDS },
+
+		// Do not explicitly signal texture data uploads (will give
+		// display corruption, but makes it possible to run with
+		// apitrace in real time). For developers only, so undocumented.
+		{ "no-flush-pbos", no_argument, 0, OPTION_NO_FLUSH_PBOS },
+
+		// Output speedcontrol debugging statistics. For developers only,
+		// so undocumented (users can get this information from Prometheus
+		// statistics, too).
+		{ "x264-speedcontrol-verbose", no_argument, 0, OPTION_X264_SPEEDCONTROL_VERBOSE },
+
+		// Deprecated variants.
+		{ "10-bit-input", no_argument, 0, OPTION_10_BIT_INPUT },
+		{ "10-bit-output", no_argument, 0, OPTION_10_BIT_OUTPUT },
+
 		{ 0, 0, 0, 0 }
 	};
 	vector<string> theme_dirs;
@@ -402,9 +422,6 @@ void parse_flags(Program program, int argc, char * const argv[])
 		case OPTION_V4L_OUTPUT:
 			global_flags.v4l_output_device = optarg;
 			break;
-		case OPTION_HTTP_UNCOMPRESSED_VIDEO:
-			global_flags.uncompressed_video_to_http = true;
-			break;
 		case OPTION_HTTP_MUX:
 			global_flags.stream_mux_name = optarg;
 			break;
@@ -438,13 +455,15 @@ void parse_flags(Program program, int argc, char * const argv[])
 			break;
 		case OPTION_RECORD_X264_VIDEO:
 			global_flags.x264_video_to_disk = true;
-			global_flags.x264_video_to_http = true;
 			break;
 		case OPTION_SEPARATE_X264_DISK_ENCODE:
 			global_flags.x264_video_to_disk = true;
 			global_flags.x264_video_to_http = true;
 			global_flags.x264_separate_disk_encode = true;
 			break;
+		case OPTION_HTTP_AV1_VIDEO:
+			global_flags.av1_video_to_http = true;
+			break;
 		case OPTION_X264_PRESET:
 			global_flags.x264_preset = optarg;
 			break;
@@ -463,12 +482,6 @@ void parse_flags(Program program, int argc, char * const argv[])
 		case OPTION_X264_CRF:
 			global_flags.x264_crf = atof(optarg);
 			break;
-		case OPTION_X264_VBV_BUFSIZE:
-			global_flags.x264_vbv_buffer_size = atoi(optarg);
-			break;
-		case OPTION_X264_VBV_MAX_BITRATE:
-			global_flags.x264_vbv_max_bitrate = atoi(optarg);
-			break;
 		case OPTION_X264_PARAM:
 			global_flags.x264_extra_param.push_back(optarg);
 			break;
@@ -487,6 +500,29 @@ void parse_flags(Program program, int argc, char * const argv[])
 		case OPTION_X264_SEPARATE_DISK_PARAM:
 			global_flags.x264_separate_disk_extra_param.push_back(optarg);
 			break;
+		case OPTION_AV1_PRESET:
+			global_flags.av1_preset = atoi(optarg);
+			break;
+		case OPTION_AV1_BITRATE:
+			global_flags.av1_bitrate = atoi(optarg);
+			break;
+		case OPTION_AV1_FPS: {
+			string str = optarg;
+			const size_t pos = str.find('/');
+			if (pos == string::npos) {
+				global_flags.av1_fps_num = stoi(str);
+				global_flags.av1_fps_den = 1;
+			} else {
+				const string num = str.substr(0, pos);
+				const string den = str.substr(pos + 1);
+				global_flags.av1_fps_num = stoi(num);
+				global_flags.av1_fps_den = stoi(den);
+			}
+			break;
+		}
+		case OPTION_AV1_PARAM:
+			global_flags.av1_extra_param.push_back(optarg);
+			break;
 		case OPTION_FLAT_AUDIO:
 			// If --flat-audio is given, turn off everything that messes with the sound,
 			// except the final makeup gain.
@@ -566,13 +602,15 @@ void parse_flags(Program program, int argc, char * const argv[])
 			global_flags.enable_quick_cut_keys = true;
 			break;
 		case OPTION_10_BIT_INPUT:
-			global_flags.ten_bit_input = true;
+			fprintf(stderr, "NOTE: --10-bit-input is a deprecated alias for --10-bit.\n");
+			global_flags.bit_depth = 10;
 			break;
 		case OPTION_10_BIT_OUTPUT:
-			global_flags.ten_bit_output = true;
-			global_flags.x264_video_to_disk = true;
-			global_flags.x264_video_to_http = true;
-			global_flags.x264_bit_depth = 10;
+			fprintf(stderr, "NOTE: --10-bit-output is a deprecated alias for --10-bit.\n");
+			global_flags.bit_depth = 10;
+			break;
+		case OPTION_10_BIT:
+			global_flags.bit_depth = 10;
 			break;
 		case OPTION_INPUT_YCBCR_INTERPRETATION: {
 			char *ptr = strchr(optarg, ',');
@@ -645,11 +683,20 @@ void parse_flags(Program program, int argc, char * const argv[])
 		}
 	}
 
-	if (global_flags.uncompressed_video_to_http &&
-	    global_flags.x264_video_to_http) {
-		fprintf(stderr, "ERROR: --http-uncompressed-video and --http-x264-video are mutually incompatible\n");
+	if (global_flags.x264_video_to_http +
+	    global_flags.av1_video_to_http > 1) {
+		fprintf(stderr, "ERROR: --http-{x264,av1}-video are mutually incompatible\n");
 		exit(1);
 	}
+	if (global_flags.bit_depth == 10) {
+		global_flags.x264_video_to_disk = true;  // No 10-bit Quick Sync support.
+		if (!global_flags.av1_video_to_http) {
+			global_flags.x264_video_to_http = true;
+		}
+	}
+	if (global_flags.x264_video_to_disk && !global_flags.av1_video_to_http) {
+		global_flags.x264_video_to_http = true;  // Quick Sync to HTTP but x264 to disk doesn't make sense.
+	}
 	if (global_flags.min_num_cards <= 0) {
 		fprintf(stderr, "ERROR: --num-cards must be at least 1\n");
 		exit(1);
@@ -759,9 +806,6 @@ void parse_flags(Program program, int argc, char * const argv[])
 			fprintf(stderr, "ERROR: --x264-bitrate and --x264-crf are mutually incompatible.\n");
 			exit(1);
 		}
-		if (global_flags.x264_vbv_max_bitrate != -1 && global_flags.x264_vbv_buffer_size != -1) {
-			fprintf(stderr, "WARNING: VBV settings are ignored with --x264-crf.\n");
-		}
 	} else if (global_flags.x264_bitrate == -1) {
 		global_flags.x264_bitrate = DEFAULT_X264_OUTPUT_BIT_RATE;
 	}
