@@ -18,6 +18,10 @@
 #include <chrono>
 #include <string>
 
+extern "C" {
+#include <libavcodec/bsf.h>
+}
+
 using namespace bmusb;
 using namespace movit;
 using namespace std;
@@ -59,10 +63,10 @@ int write_packet(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType ty
 
 }  // namespace
 
-unique_ptr<Mux> create_mux(HTTPD *httpd, AVOutputFormat *oformat, X264Encoder *x264_encoder, AudioEncoder *audio_encoder)
+unique_ptr<Mux> create_mux(HTTPD *httpd, const AVOutputFormat *oformat, X264Encoder *x264_encoder, AudioEncoder *audio_encoder)
 {
 	AVFormatContext *avctx = avformat_alloc_context();
-	avctx->oformat = oformat;
+	avctx->oformat = const_cast<decltype(avctx->oformat)>(oformat);  // const_cast is a hack to work in FFmpeg both before and after 5.0.
 
 	uint8_t *buf = (uint8_t *)av_malloc(MUX_BUFFER_SIZE);
 	avctx->pb = avio_alloc_context(buf, MUX_BUFFER_SIZE, 1, httpd, nullptr, nullptr, nullptr);
@@ -220,7 +224,7 @@ int main(int argc, char *argv[])
 
 	HTTPD httpd;
 
-	AVOutputFormat *oformat = av_guess_format(global_flags.stream_mux_name.c_str(), nullptr, nullptr);
+	const AVOutputFormat *oformat = av_guess_format(global_flags.stream_mux_name.c_str(), nullptr, nullptr);
 	assert(oformat != nullptr);
 
 	unique_ptr<AudioEncoder> audio_encoder;
@@ -230,7 +234,7 @@ int main(int argc, char *argv[])
 		audio_encoder.reset(new AudioEncoder(global_flags.stream_audio_codec_name, global_flags.stream_audio_codec_bitrate, oformat));
 	}
 
-	unique_ptr<X264Encoder> x264_encoder(new X264Encoder(oformat));
+	unique_ptr<X264Encoder> x264_encoder(new X264Encoder(oformat, /*use_separate_disk_params=*/false));
 	unique_ptr<Mux> http_mux = create_mux(&httpd, oformat, x264_encoder.get(), audio_encoder.get());
 	if (global_flags.transcode_audio) {
 		audio_encoder->add_mux(http_mux.get());
