@@ -59,7 +59,7 @@ public:
 
 	// So we never get negative dts.
 	int64_t global_delay() const {
-		return int64_t(ip_period - 1) * (TIMEBASE / MAX_FPS);
+		return int64_t(ip_period - 1) * (TIMEBASE / TYPICAL_FPS);
 	}
 
 private:
@@ -209,7 +209,7 @@ private:
 	static constexpr int initial_qp = 15;
 	static constexpr int minimal_qp = 0;
 	static constexpr int intra_period = 30;
-	static constexpr int intra_idr_period = MAX_FPS;  // About a second; more at lower frame rates. Not ideal.
+	static constexpr int intra_idr_period = TYPICAL_FPS;  // About a second; more at lower frame rates. Not ideal.
 
 	// Quality settings that are meant to be static, but might be overridden
 	// by the profile.
