@@ -49,6 +49,8 @@ enum EffectType {
 	MULTIPLY_EFFECT,
 	MIX_EFFECT,
 	LIFT_GAMMA_GAIN_EFFECT,
+	BLUR_EFFECT,
+	UNSHARP_MASK_EFFECT,
 
 	NO_EFFECT_TYPE
 };
